@@ -20,17 +20,11 @@ import (
 	"fmt"
 	"os"
 
-	"github.com/containerd/containerd/cmd/containerd/command"
-	"github.com/containerd/containerd/pkg/seed" //nolint:staticcheck // Global math/rand seed is deprecated, but still used by external dependencies
+	"github.com/containerd/containerd/v2/cmd/containerd/command"
 
-	_ "github.com/containerd/containerd/cmd/containerd/builtins"
+	_ "github.com/containerd/containerd/v2/cmd/containerd/builtins"
 )
 
-func init() {
-	//nolint:staticcheck // Global math/rand seed is deprecated, but still used by external dependencies
-	seed.WithTimeAndRand()
-}
-
 func main() {
 	app := command.App()
 	if err := app.Run(os.Args); err != nil {
