@@ -23,51 +23,40 @@ import (
 	"fmt"
 	"io"
 	"os"
+	"os/exec"
+	"runtime"
 	"testing"
 	"time"
 
-	. "github.com/containerd/containerd"
-	"github.com/containerd/containerd/defaults"
-	"github.com/containerd/containerd/errdefs"
-	"github.com/containerd/containerd/images"
-	"github.com/containerd/containerd/leases"
-	"github.com/containerd/containerd/log"
-	"github.com/containerd/containerd/log/logtest"
-	"github.com/containerd/containerd/namespaces"
-	"github.com/containerd/containerd/pkg/testutil"
-	"github.com/containerd/containerd/platforms"
-	"github.com/containerd/containerd/sys"
 	"github.com/opencontainers/go-digest"
 	"github.com/opencontainers/image-spec/identity"
-	"github.com/sirupsen/logrus"
-	exec "golang.org/x/sys/execabs"
+	"github.com/opencontainers/runtime-spec/specs-go/features"
+	"github.com/stretchr/testify/require"
+	"go.opentelemetry.io/otel"
+
+	. "github.com/containerd/containerd/v2/client"
+	"github.com/containerd/containerd/v2/core/images"
+	"github.com/containerd/containerd/v2/core/leases"
+	"github.com/containerd/containerd/v2/defaults"
+	imagelist "github.com/containerd/containerd/v2/integration/images"
+	"github.com/containerd/containerd/v2/pkg/namespaces"
+	"github.com/containerd/containerd/v2/pkg/testutil"
+	"github.com/containerd/errdefs"
+	"github.com/containerd/log"
+	"github.com/containerd/platforms"
 )
 
 var (
-	address           string
-	noDaemon          bool
-	noCriu            bool
-	supportsCriu      bool
-	testNamespace     = "testing"
-	testSnapshotter   = DefaultSnapshotter
-	ctrdStdioFilePath string
-
-	ctrd = &daemon{}
+	noDaemon     bool
+	noCriu       bool
+	supportsCriu bool
+	noShimCgroup bool
 )
 
 func init() {
-	flag.StringVar(&address, "address", defaultAddress, "The address to the containerd socket for use in the tests")
 	flag.BoolVar(&noDaemon, "no-daemon", false, "Do not start a dedicated daemon for the tests")
 	flag.BoolVar(&noCriu, "no-criu", false, "Do not run the checkpoint tests")
-}
-
-func testContext(t testing.TB) (context.Context, context.CancelFunc) {
-	ctx, cancel := context.WithCancel(context.Background())
-	ctx = namespaces.WithNamespace(ctx, testNamespace)
-	if t != nil {
-		ctx = logtest.WithT(ctx, t)
-	}
-	return ctx, cancel
+	flag.BoolVar(&noShimCgroup, "no-shim-cgroup", false, "Do not run the shim cgroup tests")
 }
 
 func TestMain(m *testing.M) {
@@ -87,7 +76,7 @@ func TestMain(m *testing.M) {
 	defer cancel()
 
 	if !noDaemon {
-		sys.ForceRemoveAll(defaultRoot)
+		_ = forceRemoveAll(defaultRoot)
 
 		stdioFile, err := os.CreateTemp("", "")
 		if err != nil {
@@ -135,14 +124,14 @@ func TestMain(m *testing.M) {
 	}
 
 	// allow comparison with containerd under test
-	log.G(ctx).WithFields(logrus.Fields{
+	log.G(ctx).WithFields(log.Fields{
 		"version":     version.Version,
 		"revision":    version.Revision,
 		"runtime":     os.Getenv("TEST_RUNTIME"),
 		"snapshotter": os.Getenv("TEST_SNAPSHOTTER"),
 	}).Info("running tests against containerd")
 
-	snapshotter := DefaultSnapshotter
+	snapshotter := defaults.DefaultSnapshotter
 	if ss := os.Getenv("TEST_SNAPSHOTTER"); ss != "" {
 		snapshotter = ss
 	}
@@ -189,7 +178,7 @@ func TestMain(m *testing.M) {
 			}
 		}
 
-		if err := sys.ForceRemoveAll(defaultRoot); err != nil {
+		if err := forceRemoveAll(defaultRoot); err != nil {
 			fmt.Fprintln(os.Stderr, "failed to remove test root dir", err)
 			os.Exit(1)
 		}
@@ -202,7 +191,7 @@ func TestMain(m *testing.M) {
 	os.Exit(status)
 }
 
-func newClient(t testing.TB, address string, opts ...ClientOpt) (*Client, error) {
+func newClient(t testing.TB, address string, opts ...Opt) (*Client, error) {
 	if testing.Short() {
 		t.Skip()
 	}
@@ -333,7 +322,7 @@ func TestImagePullAllPlatforms(t *testing.T) {
 	defer cancel()
 
 	cs := client.ContentStore()
-	img, err := client.Fetch(ctx, "k8s.gcr.io/pause:3.6")
+	img, err := client.Fetch(ctx, imagelist.Get(imagelist.Pause))
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -383,7 +372,7 @@ func TestImagePullSomePlatforms(t *testing.T) {
 
 	// Note: Must be different to the image used in TestImagePullAllPlatforms
 	// or it will see the content pulled by that, and fail.
-	img, err := client.Fetch(ctx, "k8s.gcr.io/e2e-test-images/busybox:1.29-2", opts...)
+	img, err := client.Fetch(ctx, "registry.k8s.io/e2e-test-images/busybox:1.29-2", opts...)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -432,23 +421,14 @@ func TestImagePullSomePlatforms(t *testing.T) {
 	}
 }
 
-func TestImagePullSchema1(t *testing.T) {
-	client, err := newClient(t, address)
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer client.Close()
-
-	ctx, cancel := testContext(t)
-	defer cancel()
-	schema1TestImage := "gcr.io/google_containers/pause:3.0@sha256:0d093c962a6c2dd8bb8727b661e2b5f13e9df884af9945b4cc7088d9350cd3ee"
-	_, err = client.Pull(ctx, schema1TestImage, WithPlatform(platforms.DefaultString()), WithSchema1Conversion)
-	if err != nil {
-		t.Fatal(err)
-	}
-}
-
 func TestImagePullWithConcurrencyLimit(t *testing.T) {
+	if os.Getenv("CIRRUS_CI") != "" {
+		// This test tends to fail under Cirrus CI + Vagrant due to "connection reset by peer" from
+		// pkg-containers.githubusercontent.com.
+		// Does GitHub throttle requests from Cirrus CI more compared to GitHub Actions?
+		t.Skip("unstable under Cirrus CI")
+	}
+
 	client, err := newClient(t, address)
 	if err != nil {
 		t.Fatal(err)
@@ -459,12 +439,41 @@ func TestImagePullWithConcurrencyLimit(t *testing.T) {
 	defer cancel()
 	_, err = client.Pull(ctx, testImage,
 		WithPlatformMatcher(platforms.Default()),
-		WithMaxConcurrentDownloads(2))
+		WithMaxConcurrentDownloads(2),
+	)
 	if err != nil {
 		t.Fatal(err)
 	}
 }
 
+func TestImagePullWithTracing(t *testing.T) {
+	client, err := newClient(t, address)
+	require.NoError(t, err)
+	defer client.Close()
+
+	ctx := namespaces.WithNamespace(context.Background(), "tracing")
+
+	//create in memory exporter and global tracer provider for test
+	exp, tp := newInMemoryExporterTracer()
+	//set the tracer provider global available
+	otel.SetTracerProvider(tp)
+	// Shutdown properly so nothing leaks.
+	defer func() { _ = tp.Shutdown(ctx) }()
+
+	//do an image pull which is instrumented, we should expect spans in the exporter
+	_, err = client.Pull(ctx, testImage, WithPlatformMatcher(platforms.Default()))
+	require.NoError(t, err)
+
+	err = tp.ForceFlush(ctx)
+	require.NoError(t, err)
+
+	//The span name was defined in client.pull when instrumented it
+	spanNameExpected := "pull.Pull"
+	spans := exp.GetSpans()
+	validateRootSpan(t, spanNameExpected, spans)
+
+}
+
 func TestClientReconnect(t *testing.T) {
 	t.Parallel()
 
@@ -499,26 +508,6 @@ func TestClientReconnect(t *testing.T) {
 	}
 }
 
-func createShimDebugConfig() string {
-	f, err := os.CreateTemp("", "containerd-config-")
-	if err != nil {
-		fmt.Fprintf(os.Stderr, "Failed to create config file: %s\n", err)
-		os.Exit(1)
-	}
-	defer f.Close()
-	if _, err := f.WriteString("version = 2\n"); err != nil {
-		fmt.Fprintf(os.Stderr, "Failed to write to config file %s: %s\n", f.Name(), err)
-		os.Exit(1)
-	}
-
-	if _, err := f.WriteString("[plugins.\"io.containerd.runtime.v1.linux\"]\n\tshim_debug = true\n"); err != nil {
-		fmt.Fprintf(os.Stderr, "Failed to write to config file %s: %s\n", f.Name(), err)
-		os.Exit(1)
-	}
-
-	return f.Name()
-}
-
 func TestDefaultRuntimeWithNamespaceLabels(t *testing.T) {
 	client, err := newClient(t, address)
 	if err != nil {
@@ -544,3 +533,46 @@ func TestDefaultRuntimeWithNamespaceLabels(t *testing.T) {
 		t.Error("failed to set default runtime from namespace labels")
 	}
 }
+
+func TestRuntimeInfo(t *testing.T) {
+	t.Parallel()
+
+	if runtime.GOOS == "windows" {
+		t.Skip("io.containerd.runhcs.v1 does not implement `containerd-shim-runhcs-v1.exe -info` yet")
+	}
+
+	client, err := newClient(t, address)
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer client.Close()
+
+	ctx, cancel := testContext(t)
+	defer cancel()
+
+	rti, err := client.RuntimeInfo(ctx, "", nil)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	if rti.Name == "" {
+		t.Fatal("got empty RuntimeInfo.Name")
+	}
+
+	feat, ok := rti.Features.(*features.Features)
+	if !ok {
+		t.Fatalf("expected RuntimeInfo.Features to be *features.Features, got %T", rti.Features)
+	}
+
+	var rroRecognized bool
+	for _, f := range feat.MountOptions {
+		// "rro" is recognized since runc v1.1.
+		// The functionality needs kernel >= 5.12, but `runc features` consistently include "rro" in feat.MountOptions.
+		if f == "rro" {
+			rroRecognized = true
+		}
+	}
+	if !rroRecognized {
+		t.Fatalf("expected feat.MountOptions to contain \"rro\", only got %v", feat.MountOptions)
+	}
+}
